use bytemuck::allocation::cast_slice_box;
use mechthild_core::{Scene, Shape, color::Rgba};
use pyo3::prelude::*;

#[pyclass(name = "Shape")]
struct PyShape(Shape);

#[pymethods]
impl PyShape {
    #[staticmethod]
    pub fn sphere(center: (f32, f32, f32), radius: f32) -> Self {
        Self(Shape::sphere(center.into(), radius))
    }

    #[staticmethod]
    pub fn plane(normal: (f32, f32, f32), d: f32) -> Self {
        Self(Shape::plane(normal.into(), d))
    }

    pub fn __str__(&self) -> PyResult<String> {
        Ok(format!("{:?}", self.0))
    }
}

#[pyclass(name = "Scene")]
struct PyScene(Scene);

#[pymethods]
impl PyScene {
    #[new]
    pub fn new() -> Self {
        Self(Scene::new())
    }

    pub fn render(&self, width: usize, height: usize) -> Vec<f32> {
        let result = self.0.render(width, height);

        cast_slice_box::<Rgba, f32>(result).into_vec()
    }
}

/// A Python module implemented in Rust.
#[pymodule]
fn mechthild(m: &Bound<'_, PyModule>) -> PyResult<()> {
    m.add_class::<PyShape>()?;
    m.add_class::<PyScene>()?;
    Ok(())
}
